==> awapi_perl_lib_2_5_6.tar.gz: awapi_perl_lib_2_5_6/bin/generate_classes.pl <==
#!/usr/bin/perl -w
#
# Copyright 2011, Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Wrapper script on top of SOAP::WSDL module for generating Perl classes
# based on the AdWords API WSDLs.

{
  # Patches applied to the SOAP::WSDL module to support automatic classes
  # generation for AdWords WSDLs. Patched sections are marked at start and end
  # of every block of added/changed code.
  no warnings qw(redefine);

  *SOAP::WSDL::Generator::Visitor::Typemap::set_typemap_entry = sub {
    my ($self, $value) = @_;
    my $path = join(q{/}, @{$self->get_path()});
    my $tm = $self->get_typemap();
    if ($tm->{$path} && $path =~ m/Fault\/detail\/ApiExceptionFault/) {
      return;
    }
    $tm->{$path} = $value;
  };

  *SOAP::WSDL::Generator::Visitor::Typemap::visit_XSD_ComplexType = sub {
    use Class::Std::Fast::Storable;
    my ($self, $ident, $type) = ($_[0], ident $_[0], $_[1]);
    my $variety = $type->get_variety();
    my $derivation = $type->get_derivation();
    my $content_model = $type->get_contentModel();
    return if not $variety or ($content_model eq "simpleContent");
    if (grep {$_ eq $variety} qw(all sequence choice)) {
      # Recursively going to visit child element since the type variety is
      # either all sequence choice.
      for (@{$type->get_element() || []}) {
        $_->_accept($self);
      }
    }

    # PATCH - We need to also check if the complex type has derivations and
    # include type path for all the types that derived from it.
    my $last_path_elem = pop(@{$self->get_path()});
    my $def_types = $self->get_definitions()->first_types();
    my $schema = @{$def_types->get_schema()}[1];
    my @types = @{$schema->get_type()};
    my $base_type = $type->get_name();

    if (@{$self->get_path()}[0] &&
        @{$self->get_path()}[0] eq "ApiExceptionFault") {
      @{$self->get_path()}[0] = "Fault/detail/ApiExceptionFault";
    }

    if (defined $base_type) {
      my $schemas =
          @{$self->get_definitions()->get_types()}[0]->get_schema;
      SCHEMA: foreach my $my_schema (@{$schemas}) {
        next SCHEMA if ($my_schema->isa("SOAP::WSDL::XSD::Schema::Builtin"));
        my @types = @{$my_schema->get_type()};
        TYPE: foreach my $type (@types) {
          if ($type->isa("SOAP::WSDL::XSD::ComplexType")) {
            my $type_name = $type->get_name();
            my $base = $type->get_base();
            next TYPE if !$base;
            $base =~ s{ .*: }{}xms;
            if ($base eq $base_type) {
              # Checking for infinite cycles if the type has already been mapped
              # before we skip to the next one.
              foreach my $path_elem (@{$self->get_path()}) {
                next TYPE if $path_elem eq $last_path_elem . "[$type_name]";
              }

              # In this case we generate a new path that includes the type name
              # E.G. /elem1/elem2[type]
              if ($last_path_elem =~ m/\[[^\]]+\]/) {
                $last_path_elem =~ s/\[[^\]]+\]/[${type_name}]/;
                push(@{$self->get_path()}, $last_path_elem);
              } else{
                push(@{$self->get_path()}, $last_path_elem . "[$type_name]");
              }
              my $typeclass = $self->get_resolver()->create_xsd_name($type);

              # Setting current typemap class before to allow it to be used from
              # inside _accept.
              $self->set_typemap_entry($typeclass);
              $type->_accept($self);

              # Setting it afterwards again since accept could have touch it.
              $self->set_typemap_entry($typeclass);
              pop(@{$self->get_path()});
            }
          }
        }
      }
    }

    push(@{$self->get_path()}, $last_path_elem);

    # END OF PATCH.

    return if (!$derivation);

    if ($derivation eq "restriction") {
      # Resolving the base, getting atomic type and runnning on elements.
      if (my $type_name = $type->get_base()) {
        my $subtype = $self->get_definitions()->first_types()->find_type(
            $type->expand($type_name));
        for (@{$subtype->get_element() || []}) {
          $_->_accept($self);
        }
      }
    } elsif ($derivation eq "extension") {
      # Resolving the base, getting atomic type and runnning on elements.
      while (my $type_name = $type->get_base()) {
        $type = $self->get_definitions()->first_types()->find_type(
            $type->expand($type_name));
        for (@{$type->get_element() || []}) {
          $_->_accept($self);
        }
      }
    }
  };

  *SOAP::WSDL::Base::init = sub {
    use Carp qw(croak carp confess);
    my ($self, @args) = @_;

    # PATCH Added for usage later to determine if a copy of the namespaces is
    # is required.
    my $xmlns_copied = 0;
    # END OF PATCH.
    my $value;
    foreach $value (@args) {
      croak @args if (not defined($value->{Name}));
      if ($value->{Name} =~ m{^xmlns\:}xms) {
        # PATCH - xmlns gets originally a pointer to the parent hash but if
        # it needs to be modified a copy of it is created in order to not change
        # the parent hash.
        if (not $xmlns_copied) {
          my %xmlns_copy = %{$self->get_xmlns};
          $self->set_xmlns(\%xmlns_copy);
          $xmlns_copied = 1;
        }
        # END OF PATCH.
        $self->get_xmlns->{$value->{LocalName}} = $value->{Value};
        next;
      }

      # Checking for namespae-qualified attributes.
      # neither XML Schema, nor WSDL1.1, nor the SOAP binding allow
      # namespace-qualified attribute names.
      my ($ns, $localname) = split /\|/, $value->{Name};
      if ($ns) {
        warn "found unrecognised attribute \{$ns}$localname (ignored)";
        next;
      }
      my $name = $value->{LocalName};
      my $method = "set_$name";
      $self->$method($value->{Value});
    }
    return $self;
  };

  # Re-implement in this file since it was marked private in the XSD module.
  # No changes made just renamed from _generate_filename to generate_filename.
  *SOAP::WSDL::Generator::Template::XSD::generate_filename = sub {
    my ($self, $name) = @_;
    $name =~ s{ \. }{::}xmsg;
    $name =~ s{ \- }{_}xmsg;
    $name =~ s{ :: }{/}xmsg;
    return "$name.pm";
  };

  *SOAP::WSDL::Generator::Template::XSD::generate_typemap = sub {
    my ($self, $arg_ref) = @_;
    my $visitor =
      SOAP::WSDL::Generator::Visitor::Typemap->new({
        type_prefix => $self->get_type_prefix(),
        element_prefix => $self->get_element_prefix(),
        definitions => $self->get_definitions(),
        typemap => {
          'Fault'             => 'SOAP::WSDL::SOAP::Typelib::Fault11',
          'Fault/faultcode'   => 'SOAP::WSDL::XSD::Typelib::Builtin::anyURI',
          'Fault/faultactor'  => 'SOAP::WSDL::XSD::Typelib::Builtin::token',
          'Fault/faultstring' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
          # PATCH Map our own FaultDetail object.
          'Fault/detail'      => 'Google::Ads::AdWords::FaultDetail',
          # END OF PATCH.
        },
        resolver => $self->get_name_resolver(),
      });

    use SOAP::WSDL::Generator::Iterator::WSDL11;
    my $iterator = SOAP::WSDL::Generator::Iterator::WSDL11->new({
      definitions => $self->get_definitions
    });

    for my $service (@{$self->get_definitions->get_service}) {
      $iterator->init({node => $service});
      while (my $node = $iterator->get_next()) {
        $node->_accept($visitor);
      }

      my $output = $arg_ref->{output} ? $arg_ref->{output} :
          $self->generate_filename(
              $self->get_name_resolver()->create_typemap_name($service));
      print "Creating typemap class $output\n"
          if not $SOAP::WSDL::Generator::Template::XSD::silent_of{ident $self};
      $self->_process('Typemap.tt', {
        service => $service,
        typemap => $visitor->get_typemap(),
        NO_POD  => $arg_ref->{NO_POD} ? 1 : 0,
      }, $output);
    }
  };
}

use strict;

use Getopt::Long;
use LWP::UserAgent;
use SOAP::WSDL::Base;
use SOAP::WSDL::Expat::WSDLParser;
use SOAP::WSDL::Factory::Generator;
use SOAP::WSDL::Generator::Template::XSD;
use SOAP::WSDL::Generator::Visitor::Typemap;

# Definition of services to generate classes from.
# Need to use variable interpolation with $VERSION, so don't define it with
# "use constant ...".
my %SERVICES = (
  v201109 => {
    cm => [qw(AdExtensionOverrideService AdGroupAdService
              AdGroupCriterionService AdGroupService AdParamService
              CampaignAdExtensionService CampaignCriterionService
              CampaignService CampaignTargetService ConstantDataService
              ConversionTrackerService DataService
              ExperimentService GeoLocationService
              LocationCriterionService MediaService MutateJobService
              ReportDefinitionService UserListService)],
    info => [qw(InfoService)],
    job => [qw(BulkMutateJobService)],
    o => [qw(TargetingIdeaService TrafficEstimatorService)],
    ch => [qw(CustomerSyncService)],
    mcm => [qw(AlertService ServicedAccountService)]
  }
);

# Retrieve command line options
my $server = "https://adwords.google.com";
Getopt::Long::GetOptions("server=s" => \$server);
$server = $server =~ /\/$/ ? substr($server, 0, -1) : $server;

if (-e "../lib/Google") {
  # Make a backup copy of the existing ../lib/Google just in case.
  my $now = time();
  system("cp -r ../lib/Google /tmp/Google-${now}") and die("$?\n");
}

# Iterating over every service to generate classes.
foreach my $version (keys(%SERVICES)) {
  foreach my $group (keys(%{$SERVICES{$version}})) {
    foreach my $service (@{$SERVICES{$version}{$group}}) {
      system("rm -rf ../lib/Google/Ads/AdWords/${version}/${service}/*") and
          die("$?\n");

      # Building generator options.
      my %opt = (
        url => "$server/api/adwords/${group}/${version}/${service}?wsdl",
        prefix => "My",
        attribute_prefix => "Google::Ads::AdWords::${version}::Attributes",
        type_prefix => "Google::Ads::AdWords::${version}",
        element_prefix => "Google::Ads::AdWords::${version}::${service}",
        typemap_prefix => "Google::Ads::AdWords::${version}::TypeMaps",
        interface_prefix => "Google::Ads::AdWords::${version}",
        server_prefix => undef,
        base_path => "../lib/",
        proxy => undef,
        generator => "XSD",
        server => 0,
        namespace => 0
      );
      printf "Generating classes for service %s/%s/%s\n", $group, $version,
             $service;
      printf "WSDL location: %s\n", $opt{url};

      # Proxy setup from enviroment vars.
      local $ENV{HTTP_PROXY} = $opt{proxy} if $opt{proxy};
      local $ENV{HTTPS_PROXY} = $opt{proxy} if $opt{proxy};

      # Setting up the transport.
      my $lwp = LWP::UserAgent->new($opt{keep_alive} ? (keep_alive => 1) : ());
      $lwp->env_proxy();
      $lwp->agent(qq[SOAP::WSDL $SOAP::WSDL::Expat::WSDLParser::VERSION]);

      # Instanciating the parser.
      my $parser = SOAP::WSDL::Expat::WSDLParser->new({user_agent => $lwp,});

      # Reading the WSDL.
      my $definitions = $parser->parse_uri($opt{url});

      # Instanciating the classes generator.
      my $generator = SOAP::WSDL::Factory::Generator->get_generator({
        type => $opt{"generator"}
      });
      $generator->set_attribute_prefix($opt{attribute_prefix});
      $generator->set_type_prefix($opt{type_prefix});
      $generator->set_typemap_prefix($opt{typemap_prefix});
      $generator->set_element_prefix($opt{element_prefix});
      $generator->set_interface_prefix($opt{interface_prefix});
      $generator->set_server_prefix($opt{server_prefix});
      $generator->set_OUTPUT_PATH($opt{base_path});
      $generator->set_definitions($definitions);

      # Generating classes.
      $generator->generate();
      $generator->generate_interface();
    }
  }
}

==> awapi_perl_lib_2_6_0.tar.gz: awapi_perl_lib_2_6_0/bin/generate_classes.pl <==
#!/usr/bin/perl -w
#
# Copyright 2011, Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Wrapper script on top of SOAP::WSDL module for generating Perl classes
# based on the AdWords API WSDLs.

{
  # Patches applied to the SOAP::WSDL module to support automatic classes
  # generation for AdWords WSDLs. Patched sections are marked at start and end
  # of every block of added/changed code.
  no warnings qw(redefine);

  *SOAP::WSDL::Generator::Visitor::Typemap::set_typemap_entry = sub {
    my ($self, $value) = @_;
    my $path = join(q{/}, @{$self->get_path()});
    my $tm = $self->get_typemap();
    if ($tm->{$path} && $path =~ m/Fault\/detail\/ApiExceptionFault/) {
      return;
    }
    $tm->{$path} = $value;
  };

  *SOAP::WSDL::Generator::Visitor::Typemap::visit_XSD_ComplexType = sub {
    use Class::Std::Fast::Storable;
    my ($self, $ident, $type) = ($_[0], ident $_[0], $_[1]);
    my $variety = $type->get_variety();
    my $derivation = $type->get_derivation();
    my $content_model = $type->get_contentModel();
    return if not $variety or ($content_model eq "simpleContent");
    if (grep {$_ eq $variety} qw(all sequence choice)) {
      # Recursively going to visit child element since the type variety is
      # either all sequence choice.
      for (@{$type->get_element() || []}) {
        $_->_accept($self);
      }
    }

    # PATCH - We need to also check if the complex type has derivations and
    # include type path for all the types that derived from it.
    my $last_path_elem = pop(@{$self->get_path()});
    my $def_types = $self->get_definitions()->first_types();
    my $schema = @{$def_types->get_schema()}[1];
    my @types = @{$schema->get_type()};
    my $base_type = $type->get_name();

    if (@{$self->get_path()}[0] &&
        @{$self->get_path()}[0] eq "ApiExceptionFault") {
      @{$self->get_path()}[0] = "Fault/detail/ApiExceptionFault";
    }

    if (defined $base_type) {
      my $schemas =
          @{$self->get_definitions()->get_types()}[0]->get_schema;
      SCHEMA: foreach my $my_schema (@{$schemas}) {
        next SCHEMA if ($my_schema->isa("SOAP::WSDL::XSD::Schema::Builtin"));
        my @types = @{$my_schema->get_type()};
        TYPE: foreach my $type (@types) {
          if ($type->isa("SOAP::WSDL::XSD::ComplexType")) {
            my $type_name = $type->get_name();
            my $base = $type->get_base();
            next TYPE if !$base;
            $base =~ s{ .*: }{}xms;
            if ($base eq $base_type) {
              # Checking for infinite cycles if the type has already been mapped
              # before we skip to the next one.
              foreach my $path_elem (@{$self->get_path()}) {
                next TYPE if $path_elem eq $last_path_elem . "[$type_name]";
              }

              # In this case we generate a new path that includes the type name
              # E.G. /elem1/elem2[type]
              if ($last_path_elem =~ m/\[[^\]]+\]/) {
                $last_path_elem =~ s/\[[^\]]+\]/[${type_name}]/;
                push(@{$self->get_path()}, $last_path_elem);
              } else{
                push(@{$self->get_path()}, $last_path_elem . "[$type_name]");
              }
              my $typeclass = $self->get_resolver()->create_xsd_name($type);

              # Setting current typemap class before to allow it to be used from
              # inside _accept.
              $self->set_typemap_entry($typeclass);
              $type->_accept($self);

              # Setting it afterwards again since accept could have touch it.
              $self->set_typemap_entry($typeclass);
              pop(@{$self->get_path()});
            }
          }
        }
      }
    }

    push(@{$self->get_path()}, $last_path_elem);

    # END OF PATCH.

    return if (!$derivation);

    if ($derivation eq "restriction") {
      # Resolving the base, getting atomic type and runnning on elements.
      if (my $type_name = $type->get_base()) {
        my $subtype = $self->get_definitions()->first_types()->find_type(
            $type->expand($type_name));
        for (@{$subtype->get_element() || []}) {
          $_->_accept($self);
        }
      }
    } elsif ($derivation eq "extension") {
      # Resolving the base, getting atomic type and runnning on elements.
      while (my $type_name = $type->get_base()) {
        $type = $self->get_definitions()->first_types()->find_type(
            $type->expand($type_name));
        for (@{$type->get_element() || []}) {
          $_->_accept($self);
        }
      }
    }
  };

  *SOAP::WSDL::Base::init = sub {
    use Carp qw(croak carp confess);
    my ($self, @args) = @_;

    # PATCH Added for usage later to determine if a copy of the namespaces is
    # is required.
    my $xmlns_copied = 0;
    # END OF PATCH.
    my $value;
    foreach $value (@args) {
      croak @args if (not defined($value->{Name}));
      if ($value->{Name} =~ m{^xmlns\:}xms) {
        # PATCH - xmlns gets originally a pointer to the parent hash but if
        # it needs to be modified a copy of it is created in order to not change
        # the parent hash.
        if (not $xmlns_copied) {
          my %xmlns_copy = %{$self->get_xmlns};
          $self->set_xmlns(\%xmlns_copy);
          $xmlns_copied = 1;
        }
        # END OF PATCH.
        $self->get_xmlns->{$value->{LocalName}} = $value->{Value};
        next;
      }

      # Checking for namespae-qualified attributes.
      # neither XML Schema, nor WSDL1.1, nor the SOAP binding allow
      # namespace-qualified attribute names.
      my ($ns, $localname) = split /\|/, $value->{Name};
      if ($ns) {
        warn "found unrecognised attribute \{$ns}$localname (ignored)";
        next;
      }
      my $name = $value->{LocalName};
      my $method = "set_$name";
      $self->$method($value->{Value});
    }
    return $self;
  };

  # Re-implement in this file since it was marked private in the XSD module.
  # No changes made just renamed from _generate_filename to generate_filename.
  *SOAP::WSDL::Generator::Template::XSD::generate_filename = sub {
    my ($self, $name) = @_;
    $name =~ s{ \. }{::}xmsg;
    $name =~ s{ \- }{_}xmsg;
    $name =~ s{ :: }{/}xmsg;
    return "$name.pm";
  };

  *SOAP::WSDL::Generator::Template::XSD::generate_typemap = sub {
    my ($self, $arg_ref) = @_;
    my $visitor =
      SOAP::WSDL::Generator::Visitor::Typemap->new({
        type_prefix => $self->get_type_prefix(),
        element_prefix => $self->get_element_prefix(),
        definitions => $self->get_definitions(),
        typemap => {
          'Fault'             => 'SOAP::WSDL::SOAP::Typelib::Fault11',
          'Fault/faultcode'   => 'SOAP::WSDL::XSD::Typelib::Builtin::anyURI',
          'Fault/faultactor'  => 'SOAP::WSDL::XSD::Typelib::Builtin::token',
          'Fault/faultstring' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
          # PATCH Map our own FaultDetail object.
          'Fault/detail'      => 'Google::Ads::AdWords::FaultDetail',
          # END OF PATCH.
        },
        resolver => $self->get_name_resolver(),
      });

    use SOAP::WSDL::Generator::Iterator::WSDL11;
    my $iterator = SOAP::WSDL::Generator::Iterator::WSDL11->new({
      definitions => $self->get_definitions
    });

    for my $service (@{$self->get_definitions->get_service}) {
      $iterator->init({node => $service});
      while (my $node = $iterator->get_next()) {
        $node->_accept($visitor);
      }

      my $output = $arg_ref->{output} ? $arg_ref->{output} :
          $self->generate_filename(
              $self->get_name_resolver()->create_typemap_name($service));
      print "Creating typemap class $output\n"
          if not $SOAP::WSDL::Generator::Template::XSD::silent_of{ident $self};
      $self->_process('Typemap.tt', {
        service => $service,
        typemap => $visitor->get_typemap(),
        NO_POD  => $arg_ref->{NO_POD} ? 1 : 0,
      }, $output);
    }
  };
}

use strict;

use Getopt::Long;
use LWP::UserAgent;
use SOAP::WSDL::Base;
use SOAP::WSDL::Expat::WSDLParser;
use SOAP::WSDL::Factory::Generator;
use SOAP::WSDL::Generator::Template::XSD;
use SOAP::WSDL::Generator::Visitor::Typemap;

# Definition of services to generate classes from.
# Need to use variable interpolation with $VERSION, so don't define it with
# "use constant ...".
my %SERVICES = (
  v201109 => {
    cm => [qw(AdExtensionOverrideService AdGroupAdService
              AdGroupCriterionService AdGroupService AdParamService
              CampaignAdExtensionService CampaignCriterionService
              CampaignService CampaignTargetService ConstantDataService
              ConversionTrackerService DataService
              ExperimentService GeoLocationService
              LocationCriterionService MediaService MutateJobService
              ReportDefinitionService UserListService)],
    info => [qw(InfoService)],
    job => [qw(BulkMutateJobService)],
    o => [qw(TargetingIdeaService TrafficEstimatorService)],
    ch => [qw(CustomerSyncService)],
    mcm => [qw(AlertService CreateAccountService ServicedAccountService)],
    billing => [qw(BudgetOrderService)]
  },
  v201109_1 => {
    cm => [qw(AdExtensionOverrideService AdGroupAdService
              AdGroupCriterionService AdGroupService AdParamService
              CampaignAdExtensionService CampaignCriterionService
              CampaignService CampaignTargetService ConstantDataService
              ConversionTrackerService DataService
              ExperimentService GeoLocationService
              LocationCriterionService MediaService MutateJobService
              ReportDefinitionService UserListService)],
    info => [qw(InfoService)],
    job => [qw(BulkMutateJobService)],
    o => [qw(TargetingIdeaService TrafficEstimatorService)],
    ch => [qw(CustomerSyncService)],
    mcm => [qw(AlertService CreateAccountService ServicedAccountService)],
    billing => [qw(BudgetOrderService)]
  }
);

# Retrieve command line options
my $server = "https://adwords.google.com";
Getopt::Long::GetOptions("server=s" => \$server);
$server = $server =~ /\/$/ ? substr($server, 0, -1) : $server;

if (-e "../lib/Google") {
  # Make a backup copy of the existing ../lib/Google just in case.
  my $now = time();
  system("cp -r ../lib/Google /tmp/Google-${now}") and die("$?\n");
}

# Iterating over every service to generate classes.
foreach my $version (keys(%SERVICES)) {
  foreach my $group (keys(%{$SERVICES{$version}})) {
    foreach my $service (@{$SERVICES{$version}{$group}}) {
      system("rm -rf ../lib/Google/Ads/AdWords/${version}/${service}/*") and
          die("$?\n");

      # Building generator options.
      my %opt = (
        url => "$server/api/adwords/${group}/${version}/${service}?wsdl",
        prefix => "My",
        attribute_prefix => "Google::Ads::AdWords::${version}::Attributes",
        type_prefix => "Google::Ads::AdWords::${version}",
        element_prefix => "Google::Ads::AdWords::${version}::${service}",
        typemap_prefix => "Google::Ads::AdWords::${version}::TypeMaps",
        interface_prefix => "Google::Ads::AdWords::${version}",
        server_prefix => undef,
        base_path => "../lib/",
        proxy => undef,
        generator => "XSD",
        server => 0,
        namespace => 0
      );
      printf "Generating classes for service %s/%s/%s\n", $group, $version,
             $service;
      printf "WSDL location: %s\n", $opt{url};

      # Proxy setup from enviroment vars.
      local $ENV{HTTP_PROXY} = $opt{proxy} if $opt{proxy};
      local $ENV{HTTPS_PROXY} = $opt{proxy} if $opt{proxy};

      # Setting up the transport.
      my $lwp = LWP::UserAgent->new($opt{keep_alive} ? (keep_alive => 1) : ());
      $lwp->env_proxy();
      $lwp->agent(qq[SOAP::WSDL $SOAP::WSDL::Expat::WSDLParser::VERSION]);

      # Instanciating the parser.
      my $parser = SOAP::WSDL::Expat::WSDLParser->new({user_agent => $lwp,});

      # Reading the WSDL.
      my $definitions = $parser->parse_uri($opt{url});

      # Instanciating the classes generator.
      my $generator = SOAP::WSDL::Factory::Generator->get_generator({
        type => $opt{"generator"}
      });
      $generator->set_attribute_prefix($opt{attribute_prefix});
      $generator->set_type_prefix($opt{type_prefix});
      $generator->set_typemap_prefix($opt{typemap_prefix});
      $generator->set_element_prefix($opt{element_prefix});
      $generator->set_interface_prefix($opt{interface_prefix});
      $generator->set_server_prefix($opt{server_prefix});
      $generator->set_OUTPUT_PATH($opt{base_path});
      $generator->set_definitions($definitions);

      # Generating classes.
      $generator->generate();
      $generator->generate_interface();
    }
  }
}
